{ SIZEDIR 1.1 - Thierry CORNU 1996           }
{               101611.3403@compuserve.com   }
{ CCPA, 14 rue des beaux soleils             }
{ 95520 OSNY                                 }
{ FRANCE                                     }

{ 1.1                                                                     }
{ calcul cumuls par repertoire (repertoires+ss-repertoires)               }

uses Crt,Dos;
const
  MaxFiles = 500;            {Absolute maximum number of files found in one directory}
  MaxNameLength = 30;        {Maximum length of a file name}
  DirDisplayStr : string[5] = '<dir>'; {Displayed in size column for subdirs}
type
  FileString = string[MaxNameLength]; {Room for just name and extension}
  tDirRec =                   {Stores info about each file}
    record                   {Variably sized}
      dAttr : Byte;          {File attribute}
      dName : FileString;    {File name}
      dOther : string[30];   {Other information, like size/date/time}
    end;

var
  F : Text; { fichier standard pour ecriture "ecran" }
            {  utiliser obligatoirement pour pouvoir utiliser }
            { les fonctions de redirection du DOS }
  st : string;
  SubMode, Aide : boolean;
  NumFiles : integer;
  SizeFiles : Longint;
  Niveau,Subniveau,SubNiveauShowed : byte;

const
  DosDelimSet : set of Char = ['\', ':', #0];

  function JustPathname(PathName : string) : string;
    {-Return just the drive:directory portion of a pathname}
  var
    I : Word;
  begin
    I := Succ(Word(Length(PathName)));
    repeat
      Dec(I);
    until (PathName[I] in DosDelimSet) or (I = 0);

    if I = 0 then             {nothing}
      JustPathname[0] := #0
    else
    if I = 1 then             {root directory}
      JustPathname := PathName[1]
    else
    if (PathName[I] = '\') then begin
      if PathName[Pred(I)] = ':' then {Root directory of a drive : leave backslash}
        JustPathname := Copy(PathName, 1, I)
      else                    {Subdirectory, remove the backslash}
        JustPathname := Copy(PathName, 1, Pred(I));
    end else                  {default directory of a drive}
      JustPathname := Copy(PathName, 1, I);
  end;

  function AddBackSlash(Dir : string) : string;
    {-Add a backslash to a directory name}
  begin
    if Dir[Length(Dir)] in DosDelimSet then
      AddBackSlash := Dir
    else
      AddBackSlash := Dir+'\';
  end;

  function FullPathName(FileName : string) : string;
    {-return a full pathname for FileName}
  var
    CurDir : PathStr;
    DrivePos : Byte;
    Drive : byte;
  begin
    DrivePos := Pos(':', FileName);
    if DrivePos <> 0 then begin   {drive letter specified}
      if FileName[Succ(DrivePos)] = '\' then {nothing to do}
        FullPathName := FileName
      else begin  {drive specified, but incomplete path}
        Drive:=Ord(Upcase(FileName[1]))-64;
        GetDir(Drive, CurDir);
        FullPathName := AddBackSlash(CurDir)+Copy(FileName, Succ(DrivePos), 100);
      end;
    end else begin {no drive specified}
      GetDir(0, CurDir);
      if FileName[1] = '\' then
        FullPathName := Copy(CurDir, 1, 2)+FileName
      else
        FullPathName := AddBackSlash(CurDir)+FileName;
    end;
  end;

  function Pad(St : string; Len : Byte) : string;
    {-Return a string right-padded to length len}
  var
    temp : string;
    StLen : Byte absolute St;
  begin
    if Length(St) >= Len then
      Pad := St
    else begin
      temp[0] := Chr(Len);
      Move(St[1], temp[1], StLen);
      if StLen < 255 then
        FillChar(temp[Succ(StLen)], Len-StLen, #32);
      Pad := temp;
    end;
  end;

  function SpaceStr(Len : Byte) : string;
    {-Return a string of length len filled with space #32}
  var
    St : string;
  begin
    St[0] := Chr(Len);
    FillChar(St[1], Len, #32);
    SpaceStr := St;
  end;

  function StUpcase(St:string):string;
  var i : integer;
  begin
    for i:=1 to length(St) do
      St[i]:=Upcase(St[i]);
    StUpCase:=St;
  end;

  procedure Copyright;
  begin
    Writeln;
    Writeln(F,'SizeDir V1.1 - Th CORNU 94-96');
  end;

  procedure AffAide;
  begin
    Copyright;
    writeln(F);
    Writeln(F,'SYNTAXE :  SIZEDIR [<path>] [/option]');
    writeln(F);
    Writeln(F,'options : /SX  include x subdirectories');
    Writeln(F,'          /AX  show x subdirectories ');
    Writeln(F,'          /? display this help');
    writeln(F);
    Writeln(F,'exemples: SIZEDIR \ /S1 display size of directories of root');
    Writeln(F,'          SIZEDIR WINDOWS /S99 display size of all subdirectories of WINDOWS');
    Writeln(F,'          SIZEDIR \ /S99 /A2 calc total size of all subdirectories under root');
    Writeln(F,'              but show only main directories and total.                      ');
  end;

  function FormVal(v:longint):string;
    {-format like xxx xxx xxx}
  var ST : string;
      i : integer;
  begin
    Str(v:12,ST);
    i:=Length(st)-2;
    repeat
      if i>0 then begin
        insert(' ',St,i);
      end;
      dec(i,3);
    until i<=0;
    Formval:=St;
  end;

  function FindFiles(SearchMask : string; FileAttr : Byte):longint;
    {-Add any matched files to File arrays}
  var
    Frec : SearchRec;          {Used in directory operations}
    DirRec : tDirRec;
    DT : DateTime;
    NextPos : Byte;
    TS : string[12];
    SizeF : longint;
    SizeDir : Longint;
    Taille : Longint;
  begin
    SizeF := 0;
    SizeDir := 0;
    inc(Niveau);
    FindFirst(SearchMask, FileAttr, Frec);
    while (DosError = 0) do begin
      with Frec do
        if Attr and VolumeID = 0 then
          {Matches directory type}
          if (Name <> '.') and (Name<>'..') then begin
            Inc(NumFiles);
            {Store information for display}
            with DirRec do begin
              dAttr := Attr;
              dName := Pad(JustPathName(SearchMask)+'\'+Name, MaxNameLength);
              dOther := SpaceStr(MaxNameLength+7);

              {Add size string}
              if dAttr and Directory <> Directory then
                Taille:=Size
              else begin
                {Subdirectory}
                Move(DirDisplayStr[1],
                  dOther[Length(DirDisplayStr)-2],
                  Length(DirDisplayStr));
                Taille:=0;
              end;
              SizeFiles:=SizeFiles+Taille;
              SizeF:=SizeF+Taille;
              if (dAttr and Directory = Directory) and (Niveau<=SubNiveau) then
                SizeDir:=SizeDir+
                {recursif call}
                FindFiles(AddBackSlash(JustPathName(SearchMask))+frec.name+'\*.*',AnyFile);
            end;
          end;
      FindNext(Frec);
    end;
    dec(Niveau);

    SizeDir:=SizeDir+SizeF;
    if Niveau<SubNiveauShowed then
      Writeln(F,Pad(JustPathName(SearchMask)+' ',38),
                FormVal(SizeF), '  ',
                FormVal(SizeDir),
                ' octets');
    FindFiles:=SizeDir;
  end;


var i:byte;
    err : integer;
begin
  Assign(F,''); Rewrite(F);
  DirectVideo:=false; { no direct video memory acces for }
                      { DOS redirection ( >FILE.TXT ) }
  writeln(F);
  Subniveau:=0;
  SubNiveauShowed:=255;
  Niveau:=0;
  Aide := false;
  st:='';
  for i:=1 to ParamCount do begin
    if Copy(StUpCase(ParamStr(i)),1,2)='/S' then begin
      val(Copy(ParamStr(i),3,Length(ParamStr(i))-2),Subniveau,err);
      Aide:=err<>0;
    end
    else
    if Copy(StUpCase(ParamStr(i)),1,2)='/A' then begin
      val(Copy(ParamStr(i),3,Length(ParamStr(i))-2),SubniveauShowed,err);
      Aide:=err<>0;
    end
    else
    if (ParamStr(i)='/?') or (ParamStr(i)='?') then
      Aide:=true
    else
    if Copy(ParamStr(i),1,1)='/' then Aide:=true
    else
    st:=StUpCase(ParamStr(i));
  end;
  if Aide then begin
    AffAIde;
    Halt(1);
  end;
  NumFiles:=0;
  SizeFiles:=0;
  clrscr;
  Writeln(F,'Rpertoires                            total repertoire            total');
  Writeln(F,'----------------------------------     ----------------  ----------------------');
  SizeFiles:=FindFiles(AddBackSlash(st)+'*.*',AnyFile);
  Writeln(F,'----------------------------------     ----------------  ----------------------');
  Write(F,'TOTAL de ',FullpathName(st),' = ',FormVal(SizeFiles),' octets     ');
  copyright;
  Close(F);
end.
